@@ -1,23 +1,6 @@
 /*
- * JBoss, Home of Professional Open Source.
- * Copyright 2011, Red Hat, Inc., and individual contributors
- * as indicated by the @author tags. See the copyright.txt file in the
- * distribution for a full listing of individual contributors.
- *
- * This is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this software; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
+ * Copyright The WildFly Authors
+ * SPDX-License-Identifier: Apache-2.0
  */
 
 package org.wildfly.extension.messaging.activemq;
@@ -54,7 +37,7 @@ import org.jboss.dmr.ModelNode;
 import org.jboss.dmr.ModelType;
 import org.jboss.msc.service.ServiceController;
 import org.jboss.msc.service.ServiceName;
-import org.wildfly.extension.messaging.activemq.logging.MessagingLogger;
+import org.wildfly.extension.messaging.activemq._private.MessagingLogger;
 
 /**
  * Handles operations and attribute reads supported by a ActiveMQ {@link org.apache.activemq.api.core.management.ActiveMQServerControl}.
@@ -130,16 +113,21 @@ public class ActiveMQServerControlHandler extends AbstractRuntimeOnlyHandler {
 
         final ServiceName serviceName = MessagingServices.getActiveMQServiceName(PathAddress.pathAddress(operation.get(ModelDescriptionConstants.OP_ADDR)));
 
-        if (READ_ATTRIBUTE_OPERATION.equals(operationName)) {
-            ActiveMQServer server = null;
+        if (READ_ATTRIBUTE_OPERATION.equals(operationName) || GET_ADDRESS_SETTINGS_AS_JSON.equals(operationName)) {
+            ActiveMQBroker server = null;
             if (context.getRunningMode() == RunningMode.NORMAL) {
                 ServiceController<?> service = context.getServiceRegistry(false).getService(serviceName);
                 if (service == null || service.getState() != ServiceController.State.UP) {
                     throw MessagingLogger.ROOT_LOGGER.activeMQServerNotInstalled(serviceName.getSimpleName());
                 }
-                server = ActiveMQServer.class.cast(service.getValue());
+                server = ActiveMQBroker.class.cast(service.getValue());
             }
-            handleReadAttribute(context, operation, server);
+            if (READ_ATTRIBUTE_OPERATION.equals(operationName)) {
+                handleReadAttribute(context, operation, server);
+                return;
+            }
+            String addressMatch = ADDRESS_MATCH.resolveModelAttribute(context, operation).asString();
+            context.getResult().set(server.getAddressSettingsAsJSON(addressMatch));
             return;
         }
 
@@ -203,6 +191,7 @@ public class ActiveMQServerControlHandler extends AbstractRuntimeOnlyHandler {
                 reportListOfStrings(context, list);
             } else if (LIST_PRODUCERS_INFO_AS_JSON.equals(operationName)) {
                 String json = serverControl.listProducersInfoAsJSON();
+                json = json.replace("lastProducedMessageID", "lastUUIDSent");
                 context.getResult().set(json);
             } else if (LIST_SESSIONS.equals(operationName)) {
                 String connectionID = CONNECTION_ID.resolveModelAttribute(context, operation).asString();
@@ -218,7 +207,7 @@ public class ActiveMQServerControlHandler extends AbstractRuntimeOnlyHandler {
             } else if (GET_ADDRESS_SETTINGS_AS_JSON.equals(operationName)) {
                 String addressMatch = ADDRESS_MATCH.resolveModelAttribute(context, operation).asString();
                 String json = serverControl.getAddressSettingsAsJSON(addressMatch);
-                context.getResult().set(json);
+                context.getResult().set(ManagementUtil.convertAddressSettingInfosAsJSON(json));
             } else if (FORCE_FAILOVER.equals(operationName)) {
                 serverControl.forceFailover();
                 context.getResult();
@@ -353,9 +342,10 @@ public class ActiveMQServerControlHandler extends AbstractRuntimeOnlyHandler {
                 this);
     }
 
-    private void handleReadAttribute(OperationContext context, ModelNode operation, final ActiveMQServer server) throws OperationFailedException {
+    private void handleReadAttribute(OperationContext context, ModelNode operation, final ActiveMQBroker broker) throws OperationFailedException {
         final String name = operation.require(ModelDescriptionConstants.NAME).asString();
 
+        ActiveMQServer server = broker == null ? null : ActiveMQServer.class.cast(broker.getDelegate());
         if (STARTED.getName().equals(name)) {
             boolean started = server != null ? server.isStarted() : false;
             context.getResult().set(started);
@@ -393,7 +383,7 @@ public class ActiveMQServerControlHandler extends AbstractRuntimeOnlyHandler {
         if (service == null || service.getState() != ServiceController.State.UP) {
             throw MessagingLogger.ROOT_LOGGER.activeMQServerNotInstalled(serviceName.getSimpleName());
         }
-        ActiveMQServer server = ActiveMQServer.class.cast(service.getValue());
+        ActiveMQBroker server = ActiveMQBroker.class.cast(service.getValue());
         return server.getActiveMQServerControl();
     }
 }
